﻿using System;
using System.Threading;
using System.Windows.Forms;

namespace TestProject
{
    /// <summary>
    /// 메인 폼
    /// </summary>
    public partial class MainForm : Form
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 동기화 컨텍스트
        /// </summary>
        private SynchronizationContext context;

        /// <summary>
        /// 반복 작업자
        /// </summary>
        private RepeatWorker worker;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MainForm()

        /// <summary>
        /// 생성자
        /// </summary>
        public MainForm()
        {
            InitializeComponent();

            this.context = SynchronizationContext.Current;

            this.worker = new RepeatWorker();

            this.worker.SleepTime     = 1000;
            this.worker.WorkParameter = "홍길동";
            this.worker.WorkAction    = (p) => { 
                this.context.Send
                (
                    _ =>
                    { 
                        this.listBox.Items.Add($"{DateTime.Now.ToString("HH:mm:ss")} : {p}, {this.worker.IsRunning}");
                    },
                    null
                );
                
            };

            this.runButton.Click += runButton_Click;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region 실행 버튼 클릭시 처리하기 - runButton_Click(sender, e)

        /// <summary>
        /// 실행 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void runButton_Click(object sender, EventArgs e)
        {
            if(this.runButton.Text == "실행")
            {
                this.runButton.Text = "중단";

                this.worker.Start();
            }
            else
            {
                this.runButton.Text = "실행";

                this.worker.Stop();

                Thread.Sleep(500);

                this.listBox.Items.Add($"{DateTime.Now.ToString("HH:mm:ss")} : {this.worker.WorkParameter}, {this.worker.IsRunning}");
            }
        }

        #endregion
    }
}