﻿using System;
using System.Threading;
using System.Threading.Tasks;

namespace TestProject
{
    /// <summary>
    /// 작업 반복자
    /// </summary>
    public class RepeatWorker : IDisposable
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field (Private)

        /// <summary>
        /// 취소 토큰 소스
        /// </summary>
        private CancellationTokenSource cancellationTokenSource = null;

        /// <summary>
        /// 건너뛰기 시간 (단위 : 밀리초)
        /// </summary>
        private int skipTime;

        /// <summary>
        /// 건너뛰기 시간 범위
        /// </summary>
        private TimeSpan skipTimeSpan;

        /// <summary>
        /// 휴지 시간 (단위 : 밀리초)
        /// </summary>
        private int sleepTime = 1000;

        /// <summary>
        /// 휴지 시간 범위
        /// </summary>
        private TimeSpan sleepTimeSpan;

        /// <summary>
        /// 작업 액션
        /// </summary>
        private Action<object> workAction = null;

        /// <summary>
        /// 작업 매개 변수
        /// </summary>
        private object workParameter = null;

        /// <summary>
        /// 실행 여부
        /// </summary>
        private bool isRunning = false;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 건너뛰기 시간 (단위 : 밀리초) - SkipTime

        /// <summary>
        /// 건너뛰기 시간 (단위 : 밀리초)
        /// </summary>
        public int SkipTime
        {
            get
            {
                return this.skipTime;
            }
            set
            {
                this.skipTime = value;

                this.skipTimeSpan = TimeSpan.FromMilliseconds(this.skipTime);
            }
        }

        #endregion
        #region 휴지 시간 (단위 : 밀리초) - SleepTime

        /// <summary>
        /// 휴지 시간 (단위 : 밀리초)
        /// </summary>
        public int SleepTime
        {
            get
            {
                return this.sleepTime;
            }
            set
            {
                this.sleepTime = value;

                this.sleepTimeSpan = TimeSpan.FromMilliseconds(this.sleepTime);
            }
        }

        #endregion
        #region 작업 액션 - WorkAction

        /// <summary>
        /// 작업 액션
        /// </summary>
        public Action<object> WorkAction
        {
            get
            {
                return this.workAction;
            }
            set
            {
                this.workAction = value;
            }
        }

        #endregion
        #region 작업 매개 변수 - WorkParameter

        /// <summary>
        /// 작업 매개 변수
        /// </summary>
        public object WorkParameter
        {
            get
            {
                return this.workParameter;
            }
            set
            {
                this.workParameter = value;
            }
        }

        #endregion
        #region 실행 여부 - IsRunning

        /// <summary>
        /// 실행 여부
        /// </summary>
        public bool IsRunning
        {
            get
            {
                return this.isRunning;
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - RepeatWorker()

        /// <summary>
        /// 생성자
        /// </summary>
        public RepeatWorker()
        {
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 시작하기 - Start()

        /// <summary>
        /// 시작하기
        /// </summary>
        public void Start()
        {
            this.cancellationTokenSource = new CancellationTokenSource();

            _ = ExecuteWorkAsync(this.cancellationTokenSource.Token);
        }

        #endregion
        #region 중단하기 - Stop()

        /// <summary>
        /// 중단하기
        /// </summary>
        public void Stop()
        {
            if(this.cancellationTokenSource != null)
            {
                this.cancellationTokenSource.Cancel();

                this.cancellationTokenSource.Dispose();

                this.cancellationTokenSource = null;
            }
        }

        #endregion

        #region 리소스 해제하기 - Dispose()

        /// <summary>
        /// 리소스 해제하기
        /// </summary>
        public void Dispose()
        {
            Stop();
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region 비동기 작업 실행하기 - ExecuteWorkAsync()

        /// <summary>
        /// 비동기 작업 실행하기
        /// </summary>
        /// <returns>태스크</returns>
        private Task ExecuteWorkAsync()
        {
            return Task.Run(()=> {  this.workAction(this.workParameter); });
        }

        #endregion
        #region 비동기 작업 실행하기 - ExecuteWorkAsync(cancellationToken)

        /// <summary>
        /// 비동기 작업 실행하기
        /// </summary>
        /// <param name="cancellationToken">취소 토큰</param>
        /// <returns>태스크</returns>
        private async Task ExecuteWorkAsync(CancellationToken cancellationToken)
        {
            try
            {
                this.isRunning = true;

                await Task.Delay(this.skipTimeSpan, cancellationToken);

                while(true)
                {
                    await ExecuteWorkAsync();

                    await Task.Delay(this.sleepTimeSpan, cancellationToken);
                }
            }
            catch(Exception)
            {
            }
            finally
            {
                this.isRunning = false;
            }
        }

        #endregion
    }
}